@@ -6,15 +6,21 @@ import logging
 import os
 import sys
 
-import vdb.lib.config as config
-import vdb.lib.db as dbLib
+from vdb.lib import config as config
+from vdb.lib import db as dbLib
 from vdb.lib.gha import GitHubSource
 from vdb.lib.nvd import NvdSource
 
-import depscan.lib.utils as utils
-from depscan.lib.analysis import print_results, analyse, analyse_licenses, jsonl_report
+from depscan.lib import utils as utils
+from depscan.lib.analysis import (
+    analyse,
+    analyse_licenses,
+    jsonl_report,
+    print_results,
+    suggest_version,
+)
 from depscan.lib.audit import audit, type_audit_map
-from depscan.lib.bom import get_pkg_list, create_bom
+from depscan.lib.bom import create_bom, get_pkg_list
 from depscan.lib.config import license_data_dir
 from depscan.lib.license import build_license_data, bulk_lookup
 
@@ -63,17 +69,32 @@ def build_args():
         dest="sync",
         help="Sync to receive the latest vulnerability data. Should have invoked cache first.",
     )
+    parser.add_argument(
+        "--suggest",
+        action="store_true",
+        default=False,
+        dest="suggest",
+        help="Suggest appropriate fix version for each identified vulnerability.",
+    )
+    parser.add_argument(
+        "-t",
+        "--type",
+        dest="project_type",
+        help="Override project type if auto-detection is incorrect",
+    )
     parser.add_argument(
         "--bom",
         dest="bom",
         help="UNUSED: Examine using the given Software Bill-of-Materials (SBoM) file in CycloneDX format. Use cdxgen command to produce one.",
     )
-    parser.add_argument("--src", dest="src_dir", help="Source directory", required=True)
     parser.add_argument(
+        "-i", "--src", dest="src_dir", help="Source directory", required=True
+    )
+    parser.add_argument(
+        "-o",
         "--report_file",
         dest="report_file",
         help="Report filename with directory",
-        default="reports" + os.sep + "depscan.json",
     )
     parser.add_argument(
         "--no-error",
@@ -85,31 +106,87 @@ def build_args():
     return parser.parse_args()
 
 
-def scan(db, pkg_list):
+def scan(db, pkg_list, suggest_mode):
     """
     Method to search packages in our vulnerability database
 
     :param db: Reference to db
     :param pkg_list: List of packages
     """
-    LOG.info("Scanning {} oss dependencies for issues".format(len(pkg_list)))
-    return utils.search_pkgs(db, pkg_list)
+    if not pkg_list:
+        LOG.debug("Empty package search attempted!")
+    else:
+        LOG.info("Scanning {} oss dependencies for issues".format(len(pkg_list)))
+    results, pkg_aliases = utils.search_pkgs(db, pkg_list)
+    # pkg_aliases is a dict that can be used to find the original vendor and package name
+    # This way we consistently use the same names used by the caller irrespective of how
+    # the result was obtained
+    sug_version_dict = {}
+    if suggest_mode:
+        # From the results identify optimal max version
+        sug_version_dict = suggest_version(results, pkg_aliases)
+        if sug_version_dict:
+            LOG.debug(
+                "Adjusting fix version based on the initial suggestion {}".format(
+                    sug_version_dict
+                )
+            )
+            # Recheck packages
+            sug_pkg_list = []
+            for k, v in sug_version_dict.items():
+                if not v:
+                    continue
+                vendor = ""
+                name = None
+                version = v
+                tmpA = k.split(":")
+                if len(tmpA) == 2:
+                    vendor = tmpA[0]
+                    name = tmpA[1]
+                else:
+                    name = tmpA[0]
+                # De-alias the vendor and package name
+                full_pkg = "{}:{}".format(vendor, name)
+                full_pkg = pkg_aliases.get(full_pkg, full_pkg)
+                vendor, name = full_pkg.split(":")
+                sug_pkg_list.append(
+                    {"vendor": vendor, "name": name, "version": version}
+                )
+            LOG.debug(
+                "Re-checking our suggestion to ensure there are no further vulnerabilities"
+            )
+            override_results, _ = utils.search_pkgs(db, sug_pkg_list)
+            if override_results:
+                new_sug_dict = suggest_version(override_results)
+                LOG.debug("Received override results: {}".format(new_sug_dict))
+                for nk, nv in new_sug_dict.items():
+                    sug_version_dict[nk] = nv
+    return results, pkg_aliases, sug_version_dict
 
 
-def summarise(results, report_file=None, console_print=True):
+def summarise(
+    project_type,
+    results,
+    pkg_aliases,
+    sug_version_dict,
+    report_file=None,
+    console_print=True,
+):
     """
     Method to summarise the results
+    :param project_type: Project type
     :param results: Scan or audit results
-    :param licenses_results: License scan result
+    :param pkg_aliases: Package aliases used
+    :param sug_version_dict: Dictionary containing version suggestions
     :param report_file: Output report file
     :param print: Boolean to indicate if the results should get printed to the console
     :return: Summary of the results
     """
-    if report_file:
-        jsonl_report(results, report_file)
+    if report_file and len(results):
+        jsonl_report(project_type, results, pkg_aliases, sug_version_dict, report_file)
     if console_print:
-        print_results(results)
-    summary = analyse(results)
+        print_results(project_type, results, pkg_aliases, sug_version_dict)
+    summary = analyse(project_type, results)
     return summary
 
 
@@ -117,35 +194,60 @@ def main():
     args = build_args()
     if not args.no_banner:
         print(at_logo, flush=True)
+    # Set logging level
+    if os.environ.get("SCAN_DEBUG_MODE") == "debug":
+        LOG.setLevel(logging.DEBUG)
+    src_dir = args.src_dir
+    if not args.src_dir:
+        src_dir = os.getcwd()
     db = dbLib.get()
     run_cacher = args.cache
-    reports_dir = os.path.dirname(args.report_file)
+    areport_file = (
+        args.report_file
+        if args.report_file
+        else os.path.join(src_dir, "reports", "depscan.json")
+    )
+    reports_dir = os.path.dirname(areport_file)
     # Create reports directory
     if not os.path.exists(reports_dir):
         os.makedirs(reports_dir)
     # Detect the project types and perform the right type of scan
-    project_types_list = utils.detect_project_type(args.src_dir)
+    if args.project_type:
+        project_types_list = args.project_type.split(",")
+    else:
+        project_types_list = utils.detect_project_type(src_dir)
     if len(project_types_list) > 1:
-        LOG.info("Multiple project types found: {}".format(project_types_list))
+        LOG.debug("Multiple project types found: {}".format(project_types_list))
     for project_type in project_types_list:
+        sug_version_dict = {}
+        pkg_aliases = {}
+        report_file = areport_file.replace(".json", "-{}.json".format(project_type))
         LOG.info("=" * 80)
-        bom_file = os.path.join(reports_dir, "bom-" + project_type + ".xml")
-        create_bom(project_type, bom_file, args.src_dir)
+        bom_file = os.path.join(reports_dir, "bom-" + project_type + ".json")
+        creation_status = create_bom(project_type, bom_file, src_dir)
+        if not creation_status:
+            LOG.debug("Bom file {} was not created successfully".format(bom_file))
+            continue
         LOG.debug("Scanning using the bom file {}".format(bom_file))
         pkg_list = get_pkg_list(bom_file)
         if not pkg_list:
-            LOG.warning("No packages found in the project!")
+            LOG.debug("No packages found in the project!")
+            continue
         licenses_results = bulk_lookup(
             build_license_data(license_data_dir), pkg_list=pkg_list
         )
-        analyse_licenses(licenses_results)
+        license_report_file = os.path.join(
+            reports_dir, "license-" + project_type + ".json"
+        )
+        analyse_licenses(project_type, licenses_results, license_report_file)
         if project_type in type_audit_map.keys():
             LOG.info(
                 "Performing remote audit for {} of type {}".format(
-                    args.src_dir, project_type
+                    src_dir, project_type
                 )
             )
-            results = audit(project_type, pkg_list, args.report_file)
+            LOG.debug(f"No of packages {len(pkg_list)}")
+            results = audit(project_type, pkg_list, report_file)
         else:
             if not dbLib.index_count(db["index_file"]):
                 run_cacher = True
@@ -175,12 +277,14 @@ def main():
             )
             LOG.info(
                 "Performing regular scan for {} using plugin {}".format(
-                    args.src_dir, project_type
+                    src_dir, project_type
                 )
             )
-            results = scan(db, pkg_list)
+            results, pkg_aliases, sug_version_dict = scan(db, pkg_list, args.suggest)
         # Summarise and print results
-        summary = summarise(results, args.report_file)
+        summary = summarise(
+            project_type, results, pkg_aliases, sug_version_dict, report_file, True
+        )
         if summary and not args.noerror and len(project_types_list) == 1:
             # Hard coded build break logic for now
             if summary.get("CRITICAL") > 0:
