@@ -40,14 +40,7 @@ def create_pkg_variations(pkg_dict):
     purl = pkg_dict.get("purl", "")
     if vendor:
         vendor_aliases.add(vendor)
-        # Add some common vendor aliases
-        if purl.startswith("pkg:maven") or purl.startswith("pkg:composer"):
-            vendor_aliases.add(name)
-        if purl.startswith("pkg:golang") and not name.startswith("go"):
-            vendor_aliases.add("golang")
-        if not purl.startswith("pkg:golang"):
-            vendor_aliases.add("get" + name)
-            vendor_aliases.add(name + "_project")
+        short_vendor = None
         if (
             vendor.startswith("org.")
             or vendor.startswith("io.")
@@ -57,7 +50,17 @@ def create_pkg_variations(pkg_dict):
             tmpA = vendor.split(".")
             # Automatically add short vendor forms
             if len(tmpA) > 2 and len(tmpA[1]) > 3:
-                vendor_aliases.add(tmpA[1])
+                if tmpA[1] != name:
+                    vendor_aliases.add(tmpA[1])
+                short_vendor = tmpA[1]
+        # Add some common vendor aliases
+        if purl.startswith("pkg:composer"):
+            vendor_aliases.add(name)
+        if purl.startswith("pkg:golang") and not name.startswith("go"):
+            vendor_aliases.add("golang")
+        if not purl.startswith("pkg:golang"):
+            vendor_aliases.add("get" + name)
+            vendor_aliases.add(name + "_project")
         for k, v in config.vendor_alias.items():
             if vendor.startswith(k) or k.startswith(vendor):
                 vendor_aliases.add(k)
